package cn.iselab.mooctest.site.service.impl;

import cn.iselab.mooctest.site.domainObject.ReportTask;
import cn.iselab.mooctest.site.domainObject.ReportTaskStatus;
import cn.iselab.mooctest.site.factory.ReportTaskFactory;
import cn.iselab.mooctest.site.repository.IReportTaskRepo;
import cn.iselab.mooctest.site.service.AutoReportProductService;
import cn.iselab.mooctest.site.web.data.ReportVO.ReportDataVO;
import com.google.gson.Gson;
import net.lingala.zip4j.exception.ZipException;
import net.lingala.zip4j.model.ZipParameters;
import net.lingala.zip4j.util.Zip4jConstants;
import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.mock.web.MockMultipartFile;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.AsyncResult;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.*;
import java.nio.charset.Charset;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.Future;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;

@Service
public class AutoReportProductServiceImpl implements AutoReportProductService {

    @Autowired
    private IReportTaskRepo iReportTaskRepo;

    @Autowired
    private OSSUploadServiceImpl ossUploadService;

    private static final Logger log = LoggerFactory.getLogger(Process.class.getName());

    @Value("${order.autoReport.service.path}")
    private String REPORT_SERVICE_PATH;
    @Value("${order.autoReport.service.template.path}")
    private String REPORT_SERVICE_TEMPLATE_PATH;
    @Value("${order.autoReport.upload.path}")
    private String REPORT_UPLOAD_PATH;
    @Value("${order.autoReport.builder.copyPath}")
    private String BUILDER_COPYPATH;
    @Value("${static.resource.autoReport.url}")
    private String AUTOREPORT_URL;
    @Value("${order.vulReport.json.upload.path}")
    private String REPORT_JSON_UPLOAD_PATH;
    @Value("${host.ip}")
    private String HOST_IP;

    @Override
    public ReportTask getReport(String traceId) {
        return iReportTaskRepo.getReportByTraceId(traceId);
    }

    @Async
    @Override
    public void generateReport(String traceId, String jsonData, String reportType) {
        log.info("开始做任务" + traceId);
        //存入数据库
        ReportTask reportTask = ReportTaskFactory.createReportTask();
        reportTask.setTraceId(traceId);
        reportTask.setReportType(reportType);
        reportTask.setStatus(ReportTaskStatus.GENERATING);
        Long reportTaskId = iReportTaskRepo.saveReportTask(reportTask);

        long start = System.currentTimeMillis();

        String builderPath = copyReportBuilderToPath(traceId, reportType);
        writeDataToJsonFile(jsonData, builderPath + "/" + reportType + REPORT_JSON_UPLOAD_PATH);
        System.out.println("Copy success: " + builderPath + "/"+ reportType +"/");
        boolean bale = runBuilderToProductReport("npm run build", builderPath + "/"+ reportType +"/");
        String ossDownloadUrl;
        if (bale) {
            // npm run build 执行成功
            ossDownloadUrl = reportTransferToPath(traceId, builderPath + "/"+reportType+"/");
            if(ossDownloadUrl != null && ossDownloadUrl != ""){
                // 报告zip上次成功
                String taskUrl = "http://" + HOST_IP + "/reportFile/" + traceId + "/vulReport/dist/index.html";
                // 存入数据库
                reportTask.setId(reportTaskId);
                reportTask.setTaskUrl(taskUrl);
                reportTask.setOssDownloadUrl(ossDownloadUrl);
                reportTask.setStatus(ReportTaskStatus.COMPLETED);
                iReportTaskRepo.saveReportTask(reportTask);

                long end = System.currentTimeMillis();
                log.info("完成任务"+traceId+"，耗时：" + (end - start) + "毫秒");
                return ;
            }else{
                // zip上传失败
                log.info("upload zip failed");
                reportTask.setId(reportTaskId);
                reportTask.setStatus(ReportTaskStatus.FAILED);
                iReportTaskRepo.saveReportTask(reportTask);
                return ;
            }

        }else{
            // npm run build 执行失败
            log.info("npm run build failed");
            reportTask.setId(reportTaskId);
            reportTask.setStatus(ReportTaskStatus.FAILED);
            iReportTaskRepo.saveReportTask(reportTask);
            return ;
        }
    }

    private String reportTransferToPath(String traceId, String path) {
        String autoReportPath = REPORT_UPLOAD_PATH + traceId + "/vulReport";
        String downloadPath = autoReportPath + "/report.zip";
        File uploadFile = new File(autoReportPath);
        File downloadFile = new File(downloadPath);
        boolean sign = uploadFile.mkdirs();
        log.info("------------mkdirs------------", sign);
        try {
            log.info("开始拷贝文件");
            FileUtils.copyDirectoryToDirectory(new File(path + "dist"), new File(autoReportPath));
            net.lingala.zip4j.core.ZipFile zipFile = new net.lingala.zip4j.core.ZipFile(downloadFile);
            ZipParameters parameters = new ZipParameters();
            parameters.setCompressionMethod(Zip4jConstants.COMP_DEFLATE);
            parameters.setCompressionLevel(Zip4jConstants.DEFLATE_LEVEL_NORMAL);
            zipFile.addFolder(new File(autoReportPath + "/dist"), parameters);

            // 将File类型转成MultipartFile类型
            FileInputStream inputStream = new FileInputStream(downloadFile);
            MultipartFile multipartFile = new MockMultipartFile(downloadFile.getName(), downloadFile.getName(), "", inputStream);
            log.info("----------------拷贝结束-------------------");
            return ossUploadService.uploadReportZip(multipartFile, traceId);
        } catch (IOException e) {
            log.info("----------copy error{}---------", e);
        } catch (ZipException e) {
            log.info("----------copy error{}---------", e);
        }
        return null;
    }


    @Override
    public String copyReportBuilderToPath(String  traceId, String reportTypeBuilderName) {
        File file = new File(BUILDER_COPYPATH + traceId);
        File reportFile = new File(REPORT_SERVICE_PATH + reportTypeBuilderName + "-template");
        System.out.println("Using reportFileTemplate: " + reportFile);
        String command = "cp -r " + reportTypeBuilderName + " " + BUILDER_COPYPATH + traceId + "/";
        System.out.println("Copy template report to reportBuilder: "+ command);
        if (!file.exists()) {
            log.info("-------------开始拷贝生成器---------------");
            file.mkdirs();
            try {
                Process process = Runtime.getRuntime().exec(command, null, reportFile);
                new Thread() {
                    @Override
                    public void run() {
                        BufferedReader in = new BufferedReader(new InputStreamReader(process.getInputStream()));
                        BufferedReader err = new BufferedReader(new InputStreamReader(process.getErrorStream()));
                        String line = null;
                        String errLine = null;
                        try {
                            while ((line=in.readLine())!=null||(errLine=err.readLine())!=null) {
                                if(line!=null)
                                    log.info(line);
                                if(errLine!=null)
                                    log.info(errLine);
                            }
                        } catch (IOException e) {
                            log.error("-------copyReportBuilder readInputstreamException:{}",e);
                        } finally {
                            try {
                                in.close();
                                err.close();
                            }catch (IOException e){
                                log.error("-------copyReportBuilder closeInputstreamException:{}",e);
                            }
                        }
                    }
                }.start();
                process.waitFor();
            } catch (Exception e) {
                log.info("-------------copy reportBuild error{}--------------",e);
            }
        }
        return BUILDER_COPYPATH + traceId;
    }

    public void writeDataToJsonFile(String jsonData, String path) {
        File file = new File(path, "reportData.json");
        if (!file.getParentFile().exists()) {
            file.getParentFile().mkdirs();
        }
        if (file.exists()) {
            file.delete();
        }
        try {
            file.createNewFile();
            FileUtils.writeStringToFile(file, jsonData);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public boolean runBuilderToProductReport(String command, String path) {
        File file = new File(path);
        try {
            log.info("-----------file exist:{}-------",file.exists());
            Process process = Runtime.getRuntime().exec(command, null, file);
            new Thread(){
                @Override
                public void run(){
                    BufferedReader in = new BufferedReader(new InputStreamReader(process.getInputStream()));
                    BufferedReader err = new BufferedReader(new InputStreamReader(process.getErrorStream()));
                    String line = null;
                    String errline = null;
                    try {
                        while ((line = in.readLine())!=null || (errline = err.readLine())!=null){
                            if(line!=null){
//                                log.info(line);
                            }
                            if(errline!=null) {
//                               log.info(errline);
                            }
                        }
                    }catch (IOException e){
                        log.error("-------generateReport readInputstreamException:{}",e);
                    }finally {
                        try {
                            in.close();
                            err.close();
                        }catch (IOException e){
                            log.error("-------generateReport closeInputstreamException:{}",e);
                        }
                    }
                }
            }.start();

            int status = process.waitFor();
            if (status == 0) {
//                log.info(String.valueOf(status));
                return true;
            }
            return false;
        } catch (Exception e) {
            log.error("-------------npm run error{}-----------",e);
            return false;
        }
    }

    @Override
    public String handleReportTemplate(String reportTypeName, String reportTemplateFileName) {

        reportTypeName = reportTypeName + "-template";
        // 截取报告名称
        String fileName = reportTemplateFileName.substring(0,reportTemplateFileName.indexOf('.'));
        String oldTemplateFile = REPORT_SERVICE_TEMPLATE_PATH + reportTypeName+ "/" + fileName;
//        System.out.println(oldTemplateFile);

        // 删除原本的报告模板文件
//        log.info("oldTemplateFile " + oldTemplateFile);
        File file=new File(oldTemplateFile);//路径
        deleteFile(file);
//        log.info("删除oldTemplateFile " + oldTemplateFile);

        String pathFile = REPORT_SERVICE_TEMPLATE_PATH + reportTypeName+ "/";
        String zipPath = pathFile + reportTemplateFileName;
//        System.out.println(pathFile);
//        System.out.println(zipPath);
        // 解压zip
//        log.info("解压zip " + zipPath);
        decompressZip(zipPath, pathFile + "/");
//        log.info("解压zip SUCCESS");
        return "SUCCESS";
    }

    private static void deleteFile(File file){
        if (file.isFile()){//判断是否为文件，是，则删除
            file.delete();
        }else{//不为文件，则为文件夹
            String[] childFilePath = file.list();//获取文件夹下所有文件相对路径
            if(childFilePath != null){
                for (String path:childFilePath){
                    File childFile= new File(file.getAbsoluteFile()+"/"+path);
                    deleteFile(childFile);//递归，对每个都进行判断
                }
                file.delete();
            }
        }
    }

    public boolean decompressZip(String zipPath, String descDir) {
        File zipFile = new File(zipPath);
        boolean flag = false;
        File pathFile = new File(descDir);
        if(!pathFile.exists()){
            pathFile.mkdirs();
        }
        ZipFile zip = null;
        try {
            zip = new ZipFile(zipFile, Charset.forName("gbk"));//防止中文目录，乱码
            for(Enumeration entries = zip.entries(); entries.hasMoreElements();){
                ZipEntry entry = (ZipEntry)entries.nextElement();
                String zipEntryName = entry.getName();
                InputStream in = zip.getInputStream(entry);
                //指定解压后的文件夹+当前zip文件的名称
                String outPath = (descDir+zipEntryName).replace("/", File.separator);
                //判断路径是否存在,不存在则创建文件路径
                File file = new File(outPath.substring(0, outPath.lastIndexOf(File.separator)));
                if(!file.exists()){
                    file.mkdirs();
                }
                //判断文件全路径是否为文件夹,如果是上面已经上传,不需要解压
                if(new File(outPath).isDirectory()){
                    continue;
                }
                //保存文件路径信息（可利用md5.zip名称的唯一性，来判断是否已经解压）
//                System.err.println("当前zip解压之后的路径为：" + outPath);
                OutputStream out = new FileOutputStream(outPath);
                byte[] buf1 = new byte[2048];
                int len;
                while((len=in.read(buf1))>0){
                    out.write(buf1,0,len);
                }
                in.close();
                out.close();
            }
            flag = true;
            //必须关闭，要不然这个zip文件一直被占用着，要删删不掉，改名也不可以，移动也不行，整多了，系统还崩了。
            zip.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return flag;
    }
}
