package cn.iselab.mooctest.site.service.impl;

import cn.iselab.mooctest.site.configuration.OSSConfiguration;
import cn.iselab.mooctest.site.domainObject.UploadType;
import cn.iselab.mooctest.site.exception.BaseException;
import cn.iselab.mooctest.site.service.UploadService;
import com.aliyun.oss.OSS;
import com.aliyun.oss.model.PutObjectResult;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.IOException;

/**
 * @author: Diors.Po
 * @Email: 171256175@qq.com
 * @date 2019-08-05 15:42
 */
@Slf4j
@Service
public class OSSUploadServiceImpl implements UploadService {

    @Autowired
    private OSSConfiguration ossConfig;

    @Override
    public String uploadReportZip(MultipartFile file, String fileId) {
        String fileName = UploadType.REPORT_ZIP+file.getOriginalFilename().substring(0,file.getOriginalFilename().lastIndexOf("."))+"_"+fileId+"_"
                +System.currentTimeMillis()+""
                +file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf("."), file.getOriginalFilename().length());
        return doUpload(fileName, file);
    }

    @Override
    public String uploadReportTemplate(MultipartFile file){
        String fileName = UploadType.REPORT_TEMPLATE+file.getOriginalFilename().substring(0,file.getOriginalFilename().lastIndexOf("."))+"_"
                +System.currentTimeMillis()+""
                +file.getOriginalFilename().substring(file.getOriginalFilename().lastIndexOf("."), file.getOriginalFilename().length());
        return doUpload(fileName, file);
    }

    private String doUpload(String fileName, MultipartFile file){
        OSS ossClient = ossConfig.ossClient();
        try {
            PutObjectResult result = ossClient.putObject(ossConfig.getBucketName(), fileName, file.getInputStream());
            log.info(result.toString());
            return ossConfig.getBaseUrl()+fileName;
        } catch (IOException e) {
            log.error("OSS上传出错！", e);
            throw new BaseException(e.getMessage());
        } finally {
            ossClient.shutdown();
        }
    }
}
