package cn.iselab.mooctest.site.web.ctrl;

import cn.iselab.mooctest.site.domainObject.ReportTask;
import cn.iselab.mooctest.site.domainObject.ReportTaskStatus;
import cn.iselab.mooctest.site.service.AutoReportProductService;
import cn.iselab.mooctest.site.web.data.ReportVO.AppAutoTestReportVO;
import cn.iselab.mooctest.site.web.data.ReportVO.ReportDataVO;
import cn.iselab.mooctest.site.web.data.ReportVO.StaticScanReportDataVO;
import cn.iselab.mooctest.site.web.data.ReportVO.WebData;
import org.apache.http.protocol.HTTP;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URLDecoder;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * @author guochao
 * @date 2019-11-14 22:22
 */
@RestController
public class HandleReportController {

    @Value("${order.vulReport.json.upload.path}")
    private String REPORT_JSON_UPLOAD_PATH;
    @Value("${order.autoReport.upload.path}")
    private String REPORT_UPLOAD_PATH;
    @Value("${host.ip}")
    private String HOST_IP;

    @Autowired
    private AutoReportProductService autoReportProductService;

    private static final Logger log = LoggerFactory.getLogger(Process.class.getName());


    @Value("${username:lily}")
    private String username;

    @RequestMapping("/username")
    public String get() {
        return username;
    }

    /**
     * 获取报告任务状态
     * @param traceId
     * @return
     */
    @ResponseBody
    @RequestMapping(value = "/api/report/status/{traceId}", method = RequestMethod.GET)
    public Map getReportTaskStatus(@PathVariable(value = "traceId") String traceId) {
        Map<String, Object> map = new HashMap<>();

        ReportTask reportTask = autoReportProductService.getReport(traceId);
        if(reportTask == null) {
            // 报告任务不存在
            map.put("status", ReportTaskStatus.INEXISTENCE);
            map.put("describe", "报告任务不存在");
        }else{
            // 报告任务存在
            if(reportTask.getStatus() == ReportTaskStatus.GENERATING){
                // 报告任务正在执行
                map.put("status", ReportTaskStatus.GENERATING);
                map.put("describe", "报告生成中");
            }else if(reportTask.getStatus() == ReportTaskStatus.FAILED){
                map.put("status", ReportTaskStatus.FAILED);
                map.put("describe", "报告生成失败");
            }else{
                // 报告任务执行结束，返回任务状态和url
                map.put("status", ReportTaskStatus.COMPLETED);
                map.put("describe", "报告生成结束");
                map.put("url",reportTask.getTaskUrl());
                map.put("downloadUrl", reportTask.getOssDownloadUrl());
            }
        }
        return map;
    }

    /**
     * 静态扫描任务      static-scan-report
     * web扫描任务       web-auto-test-report
     * android扫描任务   app-auto-test-report
     * @return
     */
    @ResponseBody
    @RequestMapping(value = "/api/report/generate/{reportType}", method = RequestMethod.POST)
    public Map handleStaticScanReport(@PathVariable("reportType") String reportType, HttpServletRequest request) throws IOException {
        // 读取请求内容
        BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(request.getInputStream()));
        String line = null;
        StringBuilder stringBuilder = new StringBuilder();
        while((line = bufferedReader.readLine())!=null){
            stringBuilder.append(line);
        }
        String jsonData = stringBuilder.toString();
        String traceId = generateTraceId(reportType);
        return generateReportRes(traceId, jsonData, reportType);
    }

    private String generateTraceId(String reportType) {
        String uuid = UUID.randomUUID().toString();
        return reportType.substring(0,3) + "-" + uuid;
    }


    private Map<String, Object> generateReportRes(String traceId, String jsonData, String reportType) {
        Map<String, Object> map = new HashMap<>();
        // 查询报告任务是否存在
        ReportTask reportTask = autoReportProductService.getReport(traceId);
        if(reportTask != null) {
            // 报告任务已存在
            map.put("status", ReportTaskStatus.EXISTED);
            map.put("describe", "报告任务已存在");
        }else{
            // 报告任务不存在，生成报告
            autoReportProductService.generateReport(traceId, jsonData, reportType);
            map.put("traceId", traceId);
            map.put("status", ReportTaskStatus.GENERATING);
            map.put("describe", "报告生成中");
        }
        return map;
    }
}
